const getToken = require('./getToken');
const openid = require('./strategies/openid');
const errors = require('./errors');
const { ungzip } = require('node-gzip');
const Redis = require("ioredis");

module.exports.strategies = {
  openid: openid
};

/**
 * Returns a router with two routes /login and /callback
 *
 * @param {Object} [params] The parameters object
 * @param {string} [params.getToken] A function receiving the req object and returning the token. Defaults to Authorization header.
 * @param {string} [params.strategy] A function validating the token.
 * @returns {Function} the middleware
 */
const auth = function (params) {
  if (typeof params === 'undefined') {
    params = {};
  } else if (typeof params === 'function') {
    params = { strategy: params, getToken: getToken };
  } else if (typeof params !== 'object') {
    throw new Error('expected object or function');
  }

  if (!params.getToken) {
    params.getToken = getToken;
  }

  if (!params.strategy) {
    params.strategy = openid(params);
  }

  return async (req, res, next) => {
    try {
      const token = params.getToken(req);
      req.auth = await params.strategy(token);
      if (!req.auth) {
        return errors.createInvalidTokenError(res);
      }
      if (req.auth.claims && req.auth.claims.scope) {
        const permits = await ungzip(Buffer.from(req.auth.claims.scope, 'base64'));
        req.auth.claims.scope = permits.toString();
      }
      next();
    } catch (err) {
      return errors.createInvalidTokenError(res, err.message);
    }
  };
};

/**
 * Returns a router with two routes /login and /callback
 *
 * @param {Array|String} scopes The required scopes.
 *
 * @returns {Function} the middleware
 */
const requiredScopes = function (redisClient, ...scopes) {
  const expectedScopes = Array.prototype.concat(...scopes);

  expectedScopes
    .filter((es) => typeof es !== 'string')
    .forEach((es) => {
      throw new Error('expected string got ' + typeof es);
    });

  return (req, res, next) => {
    if (!req.auth) {
      return errors.createInvalidTokenError(res);
    }

    const tokenScopes =
      req.auth.claims && typeof req.auth.claims.scope === 'string'
        ? req.auth.claims.scope.split(' ')
        : [];

    const hasExpectedScopes = expectedScopes.length ? expectedScopes.some((s) =>
      tokenScopes.includes(s)
    ) : true;

    let sessionUUID;
    if (req.auth.claims && req.auth.claims.sessionUUID) {
      sessionUUID = req.auth.claims.sessionUUID;
    }

    redisClient.get(`logout_${sessionUUID}`).then(data => {
      if (data) {
        return errors.createInvalidTokenError(res);
      }
      if (!hasExpectedScopes) {
        return errors.createInsufficientScopeError(res, expectedScopes);
      }
      next();
    }).catch(error => {
      console.log("error_in_logout_redis_check", error);
      if (!hasExpectedScopes) {
        return errors.createInsufficientScopeError(res, expectedScopes);
      }
      next();
    });
  };
};

const issuer = {
  dev: 'https://cas-api.staging.orinabiji.ge',
  staging: 'https://cas-api.staging.orinabiji.ge',
  preproduction: 'https://cas-api.preprod.orinabiji.ge',
  production: 'https://cas-api.orinabiji.ge'
}

const allAudiences = ['mobile_ios', 'admin', 'mobile_android', 'cas', 'shop', 'courier_app_n', 'courier_app_android',
  'courier_app_ios', 'erp_pocket_app_android', 'erp_pocket_app_ios', 'shop_app_android', 'shop_app_ios', 'picker_app_android', 'picker_app_ios'];

const redisHost = {
  dev: 'redis.staging.orinabiji.ge',
  staging: 'redis.staging.orinabiji.ge',
  preproduction: 'redis.preproduction.orinabiji.ge',
  production: 'redis.orinabiji.ge'
};

const redisClient = new Redis({
  // sentinels: [{
  //   host: redisHost, port: redisPort
  // }],
  // name: 'mymaster'
  host: redisHost[process.env.NODE_ENV || 'dev'],
  port: 6379
});


/**
 * Returns a router with two routes /login and /callback
 *
 * @param {Array|String} scopes The required scopes.
 *
 * @returns {Function} the middleware
 */
module.exports.checkPermits = function (permits, audiences, issuerUri) {
  if (!audiences) {
    audiences = allAudiences;
  }

  permits = !permits ? [] : permits;

  return (req, res, next) => {
    auth({
      issuerBaseURL: issuerUri || issuer[process.env.NODE_ENV || 'dev'],
      allowedAudiences: audiences
    })(req, res, () => {
      requiredScopes(redisClient, ...permits)(req, res, next);
    });
  };
};


/**
 * Returns a router with two routes /login and /callback
 *
 *
 * @returns {Function} the middleware
 */
module.exports.checkUser = function () {
  return async (req, res, next) => {
    try {
      const params = {
        issuerBaseURL: issuer[process.env.NODE_ENV || 'dev'],
        allowedAudiences: allAudiences,
        getToken
      };
      params.strategy = openid(params)
      const token = getToken(req);
      req.auth = await params.strategy(token);
      next();
    } catch (err) {
      next();
    }
  };
};

